@@ -1,5 +1,5 @@
 /* es.c - Generic code for creating an Emacspeak server
- * $Id: es.c,v 1.20 2004/02/20 23:47:27 mgorse Exp $
+ * $Id: es.c,v 1.27 2007/01/19 00:00:57 mgorse Exp $
  */
 
 #define _GNU_SOURCE
@@ -9,7 +9,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -17,6 +16,7 @@
 #include <errno.h>
 #include "es.h"
 #include <dlfcn.h>
+#include <sys/socket.h>
 
 /* This table basically stolen from Vocal-Eyes, and I am not even sure if it
  * is correct under Linux. */
@@ -411,20 +411,20 @@ void es_addtext(CLIENT *client, char *buf)
     if (q - (unsigned char *)obuf > 896)
     {
       *q = 0;
-      lang->synth->synth(lang->synth, obuf);
-      q = obuf;
+      lang->synth->synth(lang->synth, (unsigned char *)obuf);
+      q = (unsigned char *)obuf;
     }
     if (!client->punct[(int)*p]) *q++ = *p;
     else
     {
       if (q > (unsigned char *)obuf && q[-1] != ' ') *q++ = ' ';
-      strcpy(q, ascii[(int)*p]);
+      strcpy((char *)q, ascii[(int)*p]);
       while (*q) q++;
       *q++ = ' ';
     }
   }
   *q = 0;
-  lang->synth->synth(lang->synth, obuf);
+  lang->synth->synth(lang->synth, (unsigned char *)obuf);
   text_buffered = 1;
 }
 
@@ -711,7 +711,15 @@ void passthrough(char *infile, int outfd)
   while (1)
   {
     size = read(fd, buf, sizeof(buf));
-    if (size == -1) terror("read");
+    if (size == -1)
+    {
+      if (errno == EBADF)
+      {
+	/* this might really be an eof */
+	exit(0);
+      }
+      else terror("read");
+    }
     if (size == 0)
     {
       int is_fifo;
@@ -754,25 +762,36 @@ int main (int argc, char *argv[])
   int i;
   int maxclients = 0;	/* Number of clients with space allocated */
   char *infile = NULL;
+  char *context = NULL;
   int max;
   int local_fd;
   int child;
   char *input = NULL;
   int more_opts = 1;
   int debug = 0;
+  int daemon_only = 0;
 
-  while (more_opts) switch(getopt_long(argc, argv, "df:v", (struct option *)&longopts, NULL))
+  while (more_opts)
   {
-  case 'd':
-    debug = 1;
-    break;
-  case 'f':
-    input = optarg;
-    break;
-  case 'v':
-    printf("Eflite 0.3.8\n");
-    exit(0);
-  default: more_opts = 0;
+	switch(getopt_long(argc, argv, "Ddf:vc:", (struct option *)&longopts, NULL))
+	{
+	case 'D':
+	  daemon_only = 1;
+	  break;
+	case 'd':
+	  debug = 1;
+	  break;
+	case 'f':
+	  input = optarg;
+	  break;
+	case 'c':
+	  context = optarg;
+	  break;
+	case 'v':
+	  printf("Eflite 0.4.1\n");
+	  exit(0);
+	default: more_opts = 0;
+	}
   }
   /* The following allows an input file name to be specified on the command
      line.  If "-" is specified, then stdin is read from, as if no argument
@@ -780,22 +799,52 @@ int main (int argc, char *argv[])
   if (input && strcmp(input, "-") && !is_dir(input)) infile = input;
   buf = (char *)malloc(bufsize);
   if (buf == NULL) exit(1);
-  sockname = lookup_string(NULL, "socketfile");
-  if (!sockname) sockname = "/tmp/es.socket";
-  local_fd = sockconnect(sockname);
-  if (local_fd != -1) passthrough(infile, local_fd);
-  if (!debug && (child = fork()))
+  if (!context || strlen(context) == 0)
   {
-    usleep(200000);
-    local_fd = sockconnect(sockname);
-    if (local_fd == -1)
+	sockname = lookup_string(NULL, "socketfile");
+	if (!sockname) sockname = "/tmp/es.socket";
+  }
+  else
+  {
+	if (snprintf(buf, bufsize, "/tmp/es_%s.socket", context) >= bufsize)
+	{
+	  perror("Context argument too long.");
+	  exit(1);
+	}
+	sockname = strdup(buf);
+	if (!sockname)
+	{
+	  perror("Could not allocate space");
+	  exit(1);
+	}
+  }
+  local_fd = sockconnect(sockname);
+
+  if(daemon_only)
+  {
+    if(local_fd != -1)
     {
-      es_log(1 | LOG_STDERR, "Daemon not accepting connections -- exiting\n");
+      es_log(1 | LOG_STDERR, "Socket already exists.  Exiting.");
       exit(1);
     }
-    passthrough(infile, local_fd);
-    exit(0);
   }
+  else
+  {
+    if (local_fd != -1) passthrough(infile, local_fd);
+    if (!debug && (child = fork()))
+    {
+      usleep(200000);
+      local_fd = sockconnect(sockname);
+      if (local_fd == -1)
+      {
+        es_log(1 | LOG_STDERR, "Daemon not accepting connections -- exiting");
+        exit(1);
+      }
+      passthrough(infile, local_fd);
+      exit(0);
+    }
+  }
+
   punct_some = lookup_string(NULL, "punct_some");
   if (punct_some == NULL) punct_some = "@#$%^&_[]{}\\|";
   punct_all = lookup_string(NULL, "punct_all");
@@ -808,23 +857,29 @@ int main (int argc, char *argv[])
   sock = sockopen(sockname);
   if (sock == -1)
   {
-    es_log(1 | LOG_STDERR, "Error opening socket: %s\n", sockname);
+    es_log(1 | LOG_STDERR, "Error opening socket: %s", sockname);
     exit(1);
   }
   /* The following line doesn't seem to work.  Why not? */
-  fchmod(sock, 0666);
-  es_log(1, "Socket initialized\n");
+  chmod(sockname, 0666);
+  es_log(1, "Socket initialized");
   signal(SIGINT, finish);
   signal(SIGTERM, finish);
   //signal (SIGHUP, SIG_IGN);
   lang = language_open(NULL, lookup_string);
   if (lang == NULL)
   {
-    es_log(1 | LOG_STDERR, "Error initializing language\n");
+    es_log(1 | LOG_STDERR, "Error initializing language");
     exit (1);
   }
+#ifdef __CYGWIN__
+  /* Under cygwin, calling daemon() would kill the parent's stdin, too */
+  if (!debug && daemon_only) daemon(0, 0);
+#else
   if (!debug) daemon(0, 0);
+#endif
   for (i = 0; i < NPARAMS; i++) lang->synth->get_param(lang->synth, i, &default_param[i]);
+  default_param[S_VOLUME] = lookup_int("speech_volume", default_param[S_VOLUME]);
 
   for (;;)
   {
@@ -843,7 +898,7 @@ int main (int argc, char *argv[])
       {
 	client = realloc(client, ++maxclients * sizeof(CLIENT));
       }
-      es_log(1, "Accepting connection\n");
+      es_log(1, "Accepting connection");
       client[numclients++].fd = accept(sock, 0, 0);
       client_init(&client[i]);
       continue;
@@ -854,10 +909,10 @@ int main (int argc, char *argv[])
       {
 	if (handle(&client[i]))
 	{
-	  es_log(1, "Deactivating a client\n");
+	  es_log(1, "Deactivating a client");
 	  /* Deactivate client */
 	  close(client[i].fd);
-	  memcpy(client + i, client + i + 1, sizeof(CLIENT) * (--numclients - i));
+	  memmove(client + i, client + i + 1, sizeof(CLIENT) * (--numclients - i));
 	  if (numclients == 0) finish(0);
 	}
 	break;
